/**
 *  Copyright 2006 Philip Jacob Licensed under the Apache License, 
 *  Version 2.0 (the "License"); you may not use this file except in compliance with 
 *  the License. You may obtain a copy of the License at 
 *  http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or 
 *  agreed to in writing, software distributed under the License is distributed on an 
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and limitations 
 *  under the License.
 *  
 */

package com.whirlycott.s3cdnfilter;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.apache.log4j.Logger;
import org.jets3t.service.S3Service;
import org.jets3t.service.acl.AccessControlList;
import org.jets3t.service.acl.GroupGrantee;
import org.jets3t.service.acl.Permission;
import org.jets3t.service.impl.rest.httpclient.RestS3Service;
import org.jets3t.service.model.S3Object;
import org.jets3t.service.security.AWSCredentials;

/**
 * This filter automatically pushes static content to Amazon S3.
 * 
 * @author phil
 * 
 */

public class S3CDNFilter implements Filter {

	/**
	 * Logger
	 */
	private static final Logger log = Logger.getLogger(S3CDNFilter.class);

	/**
	 * If you want to disable this filter at runtime, just make this field
	 * 'false'.
	 */
	public static boolean enabled = true;

	protected S3CDNAction action;

	protected String bucket;

	protected String accessKey;

	protected String secretKey;

	/**
	 * Five threads will service requests. Why five? No reason.
	 */
	final ExecutorService executor = Executors.newFixedThreadPool(5);

	public void destroy() {
		return;
	}

	public void doFilter(final ServletRequest req, final ServletResponse res, final FilterChain chain) throws IOException, ServletException {
		// Serve the request first.
		chain.doFilter(req, res);

		// Now do stuff (partially because we need the headers from the request
		// to know things like the content type, etc.
		if (enabled && action.needsPushed(req, res)) {
			if (res.isCommitted()) {
				log.debug("Pushing object to S3");
				final S3CDNObject obj = action.prepare(req, res);
				if (obj == null) {
					log.warn(action.getClass().getCanonicalName() + " returned a null S3CDNObject");
					return;
				}
				upload(obj);
			} else {
				log.warn("Cannot handle upload to S3 because the response has not yet been committed - make this filter run later by reconfiguring your web.xml");
			}

		}

	}

	public void init(final FilterConfig _config) throws ServletException {
		log.debug("Initializing...");

		// Read parameters
		bucket = _config.getInitParameter("bucket");
		accessKey = _config.getInitParameter("accessKey");
		secretKey = _config.getInitParameter("secretKey");

		// Create an instance of the action class.
		final String value = _config.getInitParameter("action");
		try {
			final Class clazz = Class.forName(value);
			action = (S3CDNAction) clazz.newInstance();
			log.debug("S3CDN filter action specified as: " + action.getClass().getCanonicalName());

		} catch (final Exception e) {
			log.error("Filter not configured properly: " + e.getMessage());
			throw new ServletException(e);
		}

		log.debug("Filter initialized to push to: " + bucket + " using action " + action.getClass().getCanonicalName());
	}

	public void upload(final S3CDNObject obj) {

		// Upload the content to S3.
		final Runnable run = new Runnable() {

			public void run() {

				try {
					log.debug("Pushing content to S3: " + obj.getKey());
					final AWSCredentials awsCredentials = new AWSCredentials(accessKey, secretKey);
					final S3Service s3Service = new RestS3Service(awsCredentials);

					// Create an object containing a greeting string as data.
					final S3Object uploadCandidate = new S3Object(obj.getKey());

					// This becomes fully public
					final AccessControlList objectAcl = s3Service.getBucketAcl(bucket);
					objectAcl.grantPermission(GroupGrantee.ALL_USERS, Permission.PERMISSION_READ);
					uploadCandidate.setAcl(objectAcl);

					// Set important content properties.
					uploadCandidate.setContentLength(obj.getValue().length);
					final ByteArrayInputStream data = new ByteArrayInputStream(obj.getValue());
					uploadCandidate.setDataInputStream(data);

					// TODO - parameterize this
					uploadCandidate.setContentType("image/jpg");

					s3Service.putObject(bucket, uploadCandidate);
					log.debug("Successfully pushed content to " + bucket + " ; commiting local changes.");

					action.commit(obj);
					log.debug("Local changes committed: " + obj.getKey());

				} catch (final Exception e) {
					log.warn("Failure pushing to S3 for content: " + obj.getKey());
					log.warn(e.getMessage(), e);
				}
			}

		};

		// Fire off a thread.
		executor.execute(run);

	}

}
